@@ -1,7 +1,56 @@
 import numpy as np
 import h5py
 
+def read(fname,req_quants=("x1","x2")):
+    """
+    Reads an OSIRIS output data file.
+
+    Parameters
+    ----------
+    fname : str
+        The path to the data file
+    req_quants : list of str
+        The quantities required when reading RAW or TRACKS files
+    """
+    f=h5py.File(fname,"r")
+    datatype=f.attrs["TYPE"][0].decode('UTF-8')    
+    f.close()
+    data=None
+    match datatype:
+        case "grid":
+            data=grid(fname)
+        case "particles":
+            data=raw(fname,req_quants)
+        case "tracks-2":
+            data=track(fname,req_quants)
+        case _:
+             exit()
+    return data
+
+class os_data:
+    def __init__(self,datatype):
+        self._datatype=datatype
+
+    @property
+    def datatype(self):
+        return self._datatype
+
+
 class axis:
+    """
+    A class used to represent an axis object
+    ...
+
+    Attributes
+    ----------
+    nx : int
+        the number of grid poits along the axis
+    ax_arr : np.array
+        the array of points of the axis
+    name : str
+        the label of the axis
+    """
+
     def __init__(self,nx,lims,name):
         self._nx=nx
         self._name=name
@@ -19,9 +68,28 @@ class axis:
     @property
     def ax_arr(self):
         return self._ax_arr
-    
-class data:
+
+class grid(os_data):
+    """
+    A class used to represent a grid data object
+
+    ...
+
+    Attributes
+    ----------
+    dims : int
+        a formatted string to print out what the animal says
+    axis : list of axes
+        a list of axis objects containg the spatial limits of the grid
+    data : np.array
+        the grid data
+    name : str
+        the label of the quantity in the grid
+    time_s : np.float
+        the timestamp of the grid file
+    """
     def __init__(self,fname):
+        os_data.__init__(self,"grid")
         self._axis=[]
         f=h5py.File(fname,"r")
         #field
@@ -41,7 +109,7 @@ class data:
         dataname=f.attrs["NAME"][0].decode('UTF-8')+" ["+f.attrs["UNITS"][0].decode('UTF-8')+"]"
         time_s=f.attrs["TIME"][0]
         f.close()
-
+        self._dims=len(self._axis)
         self._data=dat
         self._name=dataname
         self._time_s=time_s
@@ -50,6 +118,10 @@ class data:
     def axis(self):
         return self._axis
 
+    @property
+    def dims(self):
+        return self._dims
+
     @property
     def data(self):
         return self._data
@@ -69,8 +141,23 @@ class data:
 
 
 
-class raw:
+class raw(os_data):
+    """
+    A class used to represent a particles data object
+
+    ...
+
+    Attributes
+    ----------
+    data : dictionary of np.arrays
+        a dictionary containing the required quantities
+    label : dictionary of str
+        a dictionary containing the labels of required quantities
+    time_s : np.float
+        the timestamp of the grid file
+    """
     def __init__(self,fname,req_quants):
+        os_data.__init__(self,"particles")
         self._data=dict.fromkeys(req_quants,None)
         self._label=dict.fromkeys(req_quants,None)
         f=h5py.File(fname,"r")
@@ -101,8 +188,22 @@ class raw:
         return self._time_s
 
 
-class track:
+class track(os_data):
+    """
+    A class used to represent a tracks data object
+
+    ...
+
+    Attributes
+    ----------
+    data : dictionary of lists of np.arrays
+        a dictionary containing lists the required quantities for each particle
+    label : dictionary of str
+        a dictionary containing the labels of required quantities
+    """
+
     def __init__(self,fname,req_quants):
+        os_data.__init__(self,"tracks-v2")
         self._data=dict.fromkeys(req_quants,None)
         self._label=dict.fromkeys(req_quants,None)
         f=h5py.File(file,"r")
@@ -129,10 +230,6 @@ class track:
         data=np.array(f["data"])
         print(f.attrs["QUANTS"])
         f.close()
-
-        f=h5py.File(fname,"r")
-        #field
-        f.close()
         
         try:
             for quant in req_quants:
